@@ -1,7 +1,7 @@
 import gc, sys, joblib,os, traceback
 from logging import raiseExceptions
 import pyOfficeSheet
-from os import close
+from os import close, remove
 from typing import Any
 from time import sleep
 
@@ -66,6 +66,11 @@ def pyofficelearn(screen_width,screen_height):
 
         model = load_model(filename)
 
+    def load_project_file():
+        global keras_model
+        filename, filter = QFileDialog.getOpenFileName()
+        keras_model = joblib.load(filename)
+
 
 
 
@@ -100,6 +105,10 @@ def pyofficelearn(screen_width,screen_height):
         elif current_file_name !=None:
             directory= current_file_name
 
+        directory = directory+'.polprj' if '.polprj' not in directory else directory
+
+        joblib.dump(keras_model,filename=directory,compress=9)
+
     def saveKerasModel():
         filename,filter = QFileDialog.getSaveFileName(None,'Save Model')
 
@@ -186,7 +195,6 @@ def pyofficelearn(screen_width,screen_height):
             layout.addWidget(trainButton)
 
             def keras_thread():
-                global model
 
                 def print(string):
                     label.emitAppend(string) # emit signal to change text
@@ -215,8 +223,12 @@ def pyofficelearn(screen_width,screen_height):
                             def __init__(self):
                                 self.blockType = 'Input'
                                 self.shape = keras_model['input_shape']
+                                
                             def check_submit(self):
                                 return True
+
+                            def getArgs(self):
+                                return [self.shape]
                         
                         new_inputLayer = inputlayer()
                         classLayers.insert(0,new_inputLayer)
@@ -226,9 +238,42 @@ def pyofficelearn(screen_width,screen_height):
                     for i in classLayers:
 
                         if i.blockType == 'Input':
-                            layers.append(keras.Input(shape=i.shape))
+                            shape = i.getArgs()[0]
+                            layers.append(keras.Input(shape=shape))
 
-                    classLayers.remove(new_inputLayer)
+                        elif i.blockType == 'Embedding Layer':
+                            input_dim,output_dim,initializer,regularizers,mask,inputLength, constraints= i.getArgs()
+
+                            layers.append(keras.layers.Embedding(input_dim=input_dim
+                            ,output_dim=output_dim,embeddings_initializer=initializer
+                            ,embeddings_regularizer=regularizers,mask_zero=mask,input_length=inputLength
+                            ,embeddings_constraint=constraints))
+
+                        elif i.blockType == 'Dense Layer':
+                            units,activation,bias,ki,bi,kr,br,ar,kc,bc = i.getArgs()
+
+                            layers.append(keras.layers.Dense(units=units,activation=activation
+                            ,use_bias=bias,kernel_initializer=ki,bias_initializer=bi
+                            ,kernel_regularizer=kr,bias_regularizer=br
+                            ,activity_regularizer=ar,kernel_constraint=kc,bias_constraint=bc))
+
+                        elif i.blockType == 'Dropout Layer':
+                            value = i.getArgs()[0]
+
+                            layers.append(keras.layers.Dropout(rate=value))
+
+                        elif i.blockType == 'Masking Layer':
+                            a = i.getArgs()[0]
+
+                            layers.append(keras.layers.Masking(mask_value=a))
+
+                        elif i.blockType == 'Activation Layer':
+                            a = i.getArgs()[0]
+
+                            layers.append(keras.layers.Activation(a))
+                    
+                    if 'new_inputLayer' in locals():
+                        classLayers.remove(new_inputLayer)
 
                     if keras_model['sequential']:
                         print('building Sequential model...')
@@ -257,6 +302,8 @@ def pyofficelearn(screen_width,screen_height):
                     print('generating summary...\n')
                     model.summary(print_fn=print)
 
+                    keras_model['model'] = model
+
                     trainButton.setDisabled(False)
 
                 except:
@@ -353,6 +400,17 @@ def pyofficelearn(screen_width,screen_height):
                     self.currentLayout.addWidget(block)
                     keras_model['layers'].append(block)
 
+        def addBlock(self,text,setting=False):
+            block = LayerBlockWidget(text)
+            if text == 'Sequential Model' and keras_model['sequential']==False:
+                self.currentLayout.insertWidget(1,block)
+                keras_model['sequential'] = True
+            elif text != 'Sequential Model':
+                self.currentLayout.addWidget(block)
+                keras_model['layers'].append(block)
+            if setting:
+                exec(setting,locals())
+
 
     class LayerBlockWidget(QLabel):
         'A custom widget that represent a single keras layer'
@@ -361,11 +419,17 @@ def pyofficelearn(screen_width,screen_height):
             self.index = len(keras_model['layers'])
             self.blockType = blockType
 
+            initializers = ['random_normal','random_uniform','truncated_normal','zeros','ones','glorot_normal','glorot_uniform','identity','orthogonal','constant','variance_scaling']
+            activations = ['None','relu','sigmoid','softmax','softplus','softsign','tanh','selu','elu','exponential']
+            regulars = ['None','l1','l2']
+            constraints = ['None','max_norm','min_max_norm','non_neg','unit_norm','radial_constraint']
+
             self.setFixedHeight(int(screen_height/8))
             self.setStyleSheet('background-color:#303030;color:white;')
             self.setAttribute(Qt.WA_DeleteOnClose)
             self.layout = QGridLayout()
             self.layout.setAlignment(Qt.AlignLeft)
+            self.layout.setVerticalSpacing(int(screen_height/100))
             self.setLayout(self.layout)
 
             def deleteBlock():
@@ -385,46 +449,74 @@ def pyofficelearn(screen_width,screen_height):
             self.layout.addWidget(blockLabel,0,0,1,20)
             self.layout.addWidget(closeButton,0,21,1,1)
 
+            class Label(QLabel):
+                def __init__(self, text: str):
+                    super().__init__(text)
+                    self.setAlignment(Qt.AlignRight)
+
+
             if blockType == 'Sequential Model':
                 self.setFixedHeight(int(screen_height/16))
 
             elif blockType == 'Input':
+
                 def change(index,num):
                     self.shape[index] = num
-                self.shape = ()
 
+                class customValidate(QValidator):
+                    def validate(self, arg__1: str, arg__2: int):
+                        print(arg__1)
+                        if arg__1.replace(',','').isnumeric()or arg__1 == '':
+                            return QValidator.Acceptable,arg__1, arg__2
+                        else:
+                            return QValidator.Invalid,arg__1,arg__2
+
+
+                self.shape = (0,0)
+
+                self.inputShape = QLineEdit()
+                self.inputShape.setPlaceholderText('shape of array e.g.(3,50,50)')
+                self.inputShape.setFixedWidth(int(screen_width/10))
+                self.inputShape.setValidator(customValidate())
+
+                self.layout.addWidget(QLabel('Input shape:('),1,0,1,1)
+                self.layout.addWidget(self.inputShape,1,1,1,2)
+                self.layout.addWidget(QLabel(')'+' '*200),1,3,1,6)
 
             elif blockType == 'Embedding Layer':
                 self.input_dim = QLineEdit()
                 self.input_dim.setValidator(QIntValidator())
                 self.input_dim.setMaximumWidth(int(screen_width/20))
+
                 self.output_dim = QLineEdit()
                 self.output_dim.setValidator(QIntValidator())
                 self.output_dim.setMaximumWidth(int(screen_width/20))
+
                 self.initializer = QComboBox()
-                self.initializer.addItems(['uniform'])
+                self.initializer.addItems(['uniform']+initializers)
+
                 self.regularizer = QComboBox()
-                self.regularizer.addItems(['None'])
+                self.regularizer.addItems(regulars)
                 self.mask_zero = QCheckBox('mask_zero')
                 self.input_length = QLineEdit()
                 self.input_length.setValidator(QIntValidator())
                 self.input_length.setPlaceholderText('None')
                 self.input_length.setMaximumWidth(int(screen_width/20))
                 self.constraint = QComboBox()
-                self.constraint.addItems(['None'])
+                self.constraint.addItems(constraints)
 
-                self.layout.addWidget(QLabel('input dim: '),1,0,1,1)
+                self.layout.addWidget(Label('input dim: '),1,0,1,1)
                 self.layout.addWidget(self.input_dim,1,1,1,1)
-                self.layout.addWidget(QLabel('output dim: '),1,2,1,1)
+                self.layout.addWidget(Label('output dim: '),1,2,1,1)
                 self.layout.addWidget(self.output_dim,1,3,1,1)
-                self.layout.addWidget(QLabel('  initializer: '),1,4,1,1)
+                self.layout.addWidget(Label('  initializer: '),1,4,1,1)
                 self.layout.addWidget(self.initializer,1,5,1,1)
-                self.layout.addWidget(QLabel('  regularizer: '),1,6,1,1)
+                self.layout.addWidget(Label('  regularizer: '),1,6,1,1)
                 self.layout.addWidget(self.regularizer,1,7,1,1)
                 self.layout.addWidget(self.mask_zero,2,0,1,1)
-                self.layout.addWidget(QLabel('  input Length: '),2,2,1,1)
+                self.layout.addWidget(Label('  input Length: '),2,2,1,1)
                 self.layout.addWidget(self.input_length,2,3,1,1)
-                self.layout.addWidget(QLabel('  constraint: '),2,4,1,1)
+                self.layout.addWidget(Label('  constraint: '),2,4,1,1)
                 self.layout.addWidget(self.constraint,2,5,1,2)
 
             elif blockType == 'Dropout Layer':
@@ -433,20 +525,200 @@ def pyofficelearn(screen_width,screen_height):
                 self.rate.setSingleStep(0.01)
                 self.rate.setDecimals(2)
                 self.rate.setValue(0.20)
-                self.layout.addWidget(QLabel('rate: '),1,0,1,1)
+                self.layout.addWidget(Label('rate: '),1,0,1,1)
                 self.layout.addWidget(self.rate,1,1,1,1)
 
             elif blockType == 'Dense Layer':
+                self.setFixedHeight(int(screen_height/6))
                 self.units = QLineEdit()
                 self.units.setValidator(QIntValidator())
                 self.units.setMaximumWidth(int(screen_width/20))
 
-                self.layout.addWidget(QLabel('units: '),1,0,1,1)
+                self.activation = QComboBox()
+                self.activation.addItems(activations)
+
+                self.use_bias = QCheckBox('Use bias')
+                self.use_bias.setChecked(True)
+
+                self.kernel_initializer = QComboBox()
+                self.kernel_initializer.addItems(initializers)
+                self.kernel_initializer.setCurrentText("glorot_uniform")
+
+                self.bias_initializer = QComboBox()
+                self.bias_initializer.addItems(initializers)
+                self.bias_initializer.setCurrentText('zeros')
+
+                self.kernel_regularizer = QComboBox()
+                self.kernel_regularizer.addItems(regulars)
+
+                self.bias_regularizer = QComboBox()
+                self.bias_regularizer.addItems(regulars)
+
+                self.activity_regularizer = QComboBox()
+                self.activity_regularizer.addItems(regulars)
+
+                self.kernel_constraint = QComboBox()
+                self.kernel_constraint.addItems(constraints)
+
+                self.bias_constraint = QComboBox()
+                self.bias_constraint.addItems(constraints)
+
+                self.layout.addWidget(Label('units: '),1,0,1,1)
                 self.layout.addWidget(self.units,1,1,1,1)
+                self.layout.addWidget(Label('  activation: '),1,2,1,1)
+                self.layout.addWidget(self.activation,1,3,1,1)
+                self.layout.addWidget(self.use_bias,1,4,1,1)
+                self.layout.addWidget(Label('  kernel_init: '),1,5,1,1)
+                self.layout.addWidget(self.kernel_initializer,1,6,1,1)
+                self.layout.addWidget(Label('bias_init: '),1,7,1,1)
+                self.layout.addWidget(self.bias_initializer,1,8,1,1)
+                self.layout.addWidget(Label('Regularizers :     kernel:'),2,0,1,2)
+                self.layout.addWidget(self.kernel_regularizer,2,2,1,1)
+                self.layout.addWidget(Label('    bias:'),2,3,1,1)
+                self.layout.addWidget(self.bias_regularizer,2,4,1,1)
+                self.layout.addWidget(Label('    activity:'),2,5,1,1)
+                self.layout.addWidget(self.activity_regularizer,2,6,1,1)
+                self.layout.addWidget(Label('Constraint:         kernel:'),3,0,1,2)
+                self.layout.addWidget(self.kernel_constraint,3,2,1,1)
+                self.layout.addWidget(Label('    bias:'),3,3,1,1)
+                self.layout.addWidget(self.bias_constraint,3,4,1,1)
+
+            elif blockType == 'Masking Layer':
+                self.setFixedHeight(int(screen_height/10))
+                self.mask_value= QDoubleSpinBox()
+                self.mask_value.setRange(0.0,1.0)
+                self.mask_value.setSingleStep(0.1)
+                self.layout.addWidget(QLabel('mask_value: '),1,0,1,1)
+                self.layout.addWidget(self.mask_value,1,1,1,1)
+                self.layout.addWidget(QLabel(' '*200),1,2,1,6)
+
+            elif blockType == 'Activation Layer':
+                self.setFixedHeight(int(screen_height/10))
+                self.activation = QComboBox()
+                self.activation.addItems(activations[1:])
+
+                self.layout.addWidget(QLabel('Activation: '),1,0,1,1)
+                self.layout.addWidget(self.activation,1,1,1,1)
+                self.layout.addWidget(QLabel(' '*200),1,2,1,6)
+            
+            elif blockType == 'Conv1D Layer':
+                self.setFixedHeight(int(screen_height/5))
+                self.filter = QLineEdit()
+                self.filter.setValidator(QIntValidator())
+
+                self.kernel_size = QLineEdit()
+                self.kernel_size.setValidator(QIntValidator())
+
+                self.padding = QComboBox()
+                self.padding.addItems(['valid','same','causal'])
+
+
+                self.strides = QSpinBox()
+                self.strides.setRange(1,10)
+
+                self.activation = QComboBox()
+                self.activation.addItems(activations)
+
+                self.use_bias = QCheckBox('Use bias')
+                self.use_bias.setChecked(True)
+
+                self.kernel_initializer = QComboBox()
+                self.kernel_initializer.addItems(initializers)
+                self.kernel_initializer.setCurrentText("glorot_uniform")
+
+                self.bias_initializer = QComboBox()
+                self.bias_initializer.addItems(initializers)
+                self.bias_initializer.setCurrentText('zeros')
+
+                self.kernel_regularizer = QComboBox()
+                self.kernel_regularizer.addItems(regulars)
+
+                self.bias_regularizer = QComboBox()
+                self.bias_regularizer.addItems(regulars)
+
+                self.activity_regularizer = QComboBox()
+                self.activity_regularizer.addItems(regulars)
+
+                self.kernel_constraint = QComboBox()
+                self.kernel_constraint.addItems(constraints)
+
+                self.bias_constraint = QComboBox()
+                self.bias_constraint.addItems(constraints)
+
+                self.layout.addWidget(Label('filter* : '),1,0,1,1)
+                self.layout.addWidget(self.filter,1,1,1,1)
+                self.layout.addWidget(Label('kernel_size* : '),1,2,1,1)
+                self.layout.addWidget(self.kernel_size,1,3,1,1)
+                self.layout.addWidget(Label('  kernel_init: '),1,4,1,1)
+                self.layout.addWidget(self.kernel_initializer,1,5,1,1)
+                self.layout.addWidget(Label('bias_init: '),1,6,1,1)
+                self.layout.addWidget(self.bias_initializer,1,7,1,1)
+                self.layout.addWidget(self.use_bias,2,0,1,1)
+                self.layout.addWidget(Label('Padding: '),2,1,1,1)
+                self.layout.addWidget(self.padding,2,2,1,1)
+                self.layout.addWidget(Label('strides: '),2,3,1,1)
+                self.layout.addWidget(self.strides,2,4,1,1)
+                self.layout.addWidget(Label('regularizers :     kernel:'),3,0,1,2)
+                self.layout.addWidget(self.kernel_regularizer,3,2,1,1)
+                self.layout.addWidget(Label('    bias:'),3,3,1,1)
+                self.layout.addWidget(self.bias_regularizer,3,4,1,1)
+                self.layout.addWidget(Label('    activity:'),3,5,1,1)
+                self.layout.addWidget(self.activity_regularizer,3,6,1,1)
+                self.layout.addWidget(Label('Constraint:         kernel:'),4,0,1,2)
+                self.layout.addWidget(self.kernel_constraint,4,2,1,1)
+                self.layout.addWidget(Label('    bias:'),4,3,1,1)
+                self.layout.addWidget(self.bias_constraint,4,4,1,1)
+
+                
 
         def check_submit(self): # check if every necessary field / arg is completed
             return True
 
+        def getArgs(self):
+
+            if not self.check_submit:
+                raise Exception('not all arguments filled')
+
+            args = []
+            
+            if self.blockType == 'Input':
+                args.append((int(i) for i in self.inputShape.text().split(',')))
+
+            elif self.blockType == 'Embedding Layer':
+                args.append(int(self.input_dim.text()))
+                args.append(int(self.output_dim.text()))
+                args.append(self.initializer.currentText())
+                args.append(self.regularizer.currentText())
+                args.append(self.mask_zero.isChecked())
+                args.append(None if self.input_length.text()=='' else self.input_length.text())
+                args.append(self.constraint.currentText())
+
+            elif self.blockType == 'Dense Layer':
+                args.append(int(self.units.text()))
+                args.append(self.activation.currentText())
+                args.append(self.use_bias.isChecked())
+                args.append(self.kernel_initializer.currentText())
+                args.append(self.bias_initializer.currentText())
+                args.append(self.kernel_regularizer.currentText())
+                args.append(self.bias_regularizer.currentText())
+                args.append(self.activity_regularizer.currentText())
+                args.append(self.kernel_constraint.currentText())
+                args.append(self.bias_constraint.currentText())
+
+            elif self.blockType == 'Dropout Layer':
+                args.append(self.rate.value())
+
+            elif self.blockType == 'Masking Layer':
+                args.append(self.mask_value.value())
+
+            elif self.blockType == 'Activation Layer':
+                args.append(self.activation.currentText())
+
+            args = [None if i == 'None' else i for i in args]
+
+            return args
+            
+
     class ModelCompilerBlock(QLabel):
         'A custom Widget that represent the Model.compile function'
         def __init__(self):
@@ -582,8 +854,11 @@ def pyofficelearn(screen_width,screen_height):
     # menu
 
     left_menu_mainWidget = QWidget()
+    scroll = QScrollArea()
+    scroll.setWidgetResizable(True)
+    scroll.setWidget(left_menu_mainWidget)
     #left_menu_mainWidget.setStyleSheet('background-color:#303030;')
-    left_menu_layout.addWidget(left_menu_mainWidget,0,1,10,6)
+    left_menu_layout.addWidget(scroll,0,1,10,6)
     
 
     # first menu layou
@@ -612,6 +887,44 @@ def pyofficelearn(screen_width,screen_height):
         label.setToolTip(y)
         left_menu_mainlayout.addWidget(label)
 
+    def expend_shrink(button,widget):
+        text = button.text()
+        if '▶' in text:
+            text = text.replace('▶','▼')
+        else:
+            text = text.replace('▼', '▶')
+
+        button.setText(text)
+
+        if widget.isHidden():
+            widget.show()
+        else:
+            widget.hide()
+
+
+    conv_layers = {'Conv1D Layer':'1D convolution layer'}
+    conv_widget = QWidget()
+    conv_layout = QVBoxLayout()
+    conv_widget.setLayout(conv_layout)
+    conv_widget.hide()
+
+    for x,y in conv_layers.items():
+        label = DraggableLabel(x)
+        label.setToolTip(y)
+        conv_layout.addWidget(label)
+
+    cov_button = QPushButton('▶ Convolution Layers')
+    cov_button.setStyleSheet('text-align:left;font-size: 20px')
+    cov_button.setFlat(True)
+    cov_button.clicked.connect(lambda:expend_shrink(cov_button,conv_widget))
+    left_menu_mainlayout.addWidget(cov_button)
+    left_menu_mainlayout.addWidget(conv_widget)
+
+
+    
+
+
+
 
     mainLayout.addWidget(left_menu_widget)
 
@@ -730,7 +1043,7 @@ for more information, visit https://github.com/YC-Lammy/py-office-learn
     plt_setting = {'set':False}
     settings = {}
     prjdict = {}
-    keras_model = {'sequential':False,'layers':[],'compiler':None,'trainer':None,'input_shape':(1,)}
+    keras_model = {'sequential':False,'layers':[],'compiler':None,'trainer':None,'input_shape':(1,),'model':None}
 
 
     def closeEventHandler(event): # this function is called when user tries to close app, line 559
